///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2010 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Mar 01, 2010	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import java.util.Vector;

import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbFieldType;
import com.tietoenator.trip.jxp.ccl.TdbKernelWindow;
import com.tietoenator.trip.jxp.ccl.TdbTerm;
import com.tietoenator.trip.jxp.ccl.TdbTermList;
import com.tietoenator.trip.jxp.ccl.TdbTermTree;
import com.tietoenator.trip.jxp.ccl.TdbTreeEntry;
import com.tietoenator.trip.jxp.data.TdbField;
import com.tietoenator.trip.jxp.data.TdbFieldFetchType;
import com.tietoenator.trip.jxp.data.TdbFieldTemplate;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.data.TdbSearch;
import com.tietoenator.trip.jxp.data.TdbSearchSet;
import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.database.TdbFieldDesign;
import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * Sample program for TRIPnxp that shows how to use the TdbSearch class
 * to search, maintain a search history and retrieve results as a TdbRecord
 * collection or output format report.
 */
public class Search
{

    private TdbSession m_session;
    private TdbSearch m_search;
    private TdbDatabaseDesign m_db;

    public Search() throws TdbException
    {
        Console stdin = new Console();

        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** Example CCL Interpreter Using TdbSearch ****", false);

        m_search = new TdbSearch(m_session);
        if (stdin.readint("Custom retrieval template? (1|0)", 0, true) != 0)
        {
            m_search.setAutomaticRetrievalTemplate(true);
        }
        else
        {
            m_search.setAutomaticRetrievalTemplate(false);
        }

        // Show what we are connected to.
        System.out.println();
        System.out.println("Connected to TRIP " + m_session.getVersion() + " using the " +
                m_session.getCharacterSet() + " [" + m_session.getCharacterSetName() + "] character set.");
        System.out.println();

        System.out.println("Type GETROWS [FROMIDX] [TOIDX] to retrieve data using a template.");
        System.out.println();
    }

    /**
     * Close the TdbSearch instance and log out
     * @throws TdbException
     */
    public void close()
    {
        try { if (m_search != null) m_search.close(); }
        catch (TdbException ignore) {}
        finally { m_search = null; }

        try { if (m_session != null) m_session.logout(); }
        catch (TdbException ignore) {}
        finally { m_session = null; }
    }

    /**
     * Print search set information (history entry) to the console.
     * @param searchSet Search set to display info about.
     */
    private void printHistory(TdbSearchSet searchSet)
    {
        StringBuilder sbLine = new StringBuilder();
        sbLine.append("S=" + Integer.toString(searchSet.getSearchId()));
        while (sbLine.length() < 8)
        {
            sbLine.append(' ');
        }
        sbLine.append("<");
        sbLine.append(Integer.toString(searchSet.getRecordCount()));
        sbLine.append(">");
        while (sbLine.length() < 18)
        {
            sbLine.append(' ');
        }
        sbLine.append(searchSet.getParsedCommand());
        System.out.println(sbLine.toString());
    }
    // Utility function used to recurse through a tree of thesaurus terms
    private static final String spaces = "                                                      ";

    private void printThesTerm(TdbTreeEntry entry, int indent)
    {
        System.out.println(spaces.substring(0, indent) + entry.getName());
        for (TdbTreeEntry child : entry.children())
        {
            printThesTerm(child, indent + 2);
        }
    }

    /**
     * Create a custom retrieval template.
     *
     * A real-life application would create a custom templates for each
     * database used, and assign them to the TdbSearchSet objects as
     * required. Automatic templates are only recommended if the application
     * really is interested in getting all the data from each record.
     *
     * @return A TdbRecord representing a custom retrieval template.
     * @throws TdbException
     */
    private TdbRecord createRetrievalTemplate() throws TdbException
    {
        if (m_db == null || (m_db != null && !m_db.getName().equals(m_search.getCurrentBase())))
        {
            m_db = new TdbDatabaseDesign(m_session);
            m_db.get(m_search.getCurrentBase());
        }

        TdbRecord r = new TdbRecord(m_session, m_db, false);
        for (int i = 0; i < m_db.getFieldCount(); i++)
        {
            TdbFieldDesign f = m_db.getFieldByIndex(i);

            // Only fetch top-level non-string fields for XML databases,
            // because otherwise it'll be WAY to much data...
            if (m_db.isXmlEnabled())
            {
                if (f.getName().startsWith("D_") && f.getTypeId() != TdbFieldType.StringField)
                {
                    if (f.getTypeId() == TdbFieldType.TextField)
                    {
                        r.addToTemplate(new TdbFieldTemplate(f.getName(), TdbFieldFetchType.FocusOrExtract, 80));
                    }
                    else
                    {
                        r.addToTemplate(f.getName());
                    }
                }
            }
            // Only add structured fiels and summaries of text fields
            // from the head record. Omit string fields and part records.
            else if (f.getTypeId() != TdbFieldType.StringField && f.isPartField() == false)
            {
                if (f.getTypeId() == TdbFieldType.TextField)
                {
                    r.addToTemplate(new TdbFieldTemplate(f.getName(), TdbFieldFetchType.FocusOrExtract, 80));
                }
                else
                {
                    r.addToTemplate(f.getName());
                }
            }
        }

        return r;
    }

    /**
     * Utility method to split a string with white space as delimiter.
     * @param str String to tokenize.
     * @return Array with tokens
     */
    private String[] tokenize(String str)
    {
        Vector<String> tokenVector = new Vector<String>();
        String[] candidateTokens = str.trim().split(" ");
        for (String t : candidateTokens)
        {
            if (t.length() > 0)
            {
                tokenVector.add(t);
            }
        }

        String[] tokens = new String[tokenVector.size()];
        return tokenVector.toArray(tokens);
    }

    /**
     * Handle a CCL statement entered on the command line.
     * @param stmt Statement to handle
     * @return True to exit the application, and false to continue.
     */
    public boolean handleStatement(String stmt)
    {
        int i;
        String strCmd = "";
        boolean finished = false;

        // We get a null statement for an empty line.
        if (null == stmt)
            return false;

        try
        {
            TdbSearchSet searchSet = m_search.getLastSearchSet();

            i = stmt.indexOf(' ');
            if (i > 0)
                strCmd = stmt.substring(0, i).trim();
            else
                strCmd = stmt.trim();

            // Special client-side command to fetch data using
            // a retrieval template.
            if (searchSet != null && strCmd.equalsIgnoreCase("GETROWS"))
            {
                // If automatic retrieval templates are not used, make sure
                // that we have a custom one.
                if (m_search.getAutomaticRetrievalTemplate() == false)
                {
                    searchSet.setRetrievalTemplate(createRetrievalTemplate());
                }

                int nStart = 0;
                int nStop = searchSet.getRecordCount() - 1;

                String[] tokenized = tokenize(stmt);
                if (tokenized.length > 1)
                {
                    nStart = Integer.parseInt(tokenized[1]);
                }
                if (tokenized.length > 2)
                {
                    nStop = Integer.parseInt(tokenized[2]);
                }

                for (i = nStart; i <= nStop && i < searchSet.getRecordCount(); i++)
                {
                    TdbRecord record = searchSet.getRecord(i);

                    // Only the head record fields.
                    System.out.println("\nRecord " + record.getRecordId() + "\n");
                    for (TdbField field : record.getHead().fields())
                    {
                        System.out.println(field.getName() + ": " + field.toString());
                    }
                }

                return false;
            }

            // Execute the CCL statement
            m_search.resetMessages();
            m_search.execute(stmt);

            // Print messages
            for (String msg : m_search.messageList())
            {
                System.out.println(msg);
            }

            // Post-process the statement
            switch (m_search.getCommandType())
            {
                case Abort:
                    m_session.logout();
                    finished = true;
                    break;

                case HistoryUpdate:
                    // Any command that generates a search set ends up here.
                    printHistory(m_search.getLastSearchSet());
                    break;

                case HistoryReplace:
                    // Commands like DELETE and RENUM are handled here.
                    for (TdbSearchSet ss : m_search)
                        printHistory(ss);

                    break;

                case TermList:
                    {
                        TdbTermList list = m_search.getTermList();
                        i = 1;
                        System.out.println("" + list.size() + " terms");
                        for (TdbTerm term : list)
                        {
                            System.out.println("T=" + i++ + "\t<" + term.getRecordCount() + ">\t" + term.getTerm());
                        }
                    }
                    break;

                case TermTree:
                    {
                        TdbTermTree tree = m_search.getTermTree();
                        System.out.println("" + tree.getRootCount() + " main terms.\n");
                        for (TdbTreeEntry entry : tree.roots())
                        {
                            printThesTerm(entry, 0);
                        }
                    }
                    break;

                // Commands that produce output (SHOW, MORE, BACK, NEXT, PREV, etc.)
                case Output:
                    {
                        TdbKernelWindow w = m_search.getAffectedWindow();
                        System.out.println(w.toString());
                        if (!w.isAtBottom())
                        {
                            System.out.println("Type More to continue");
                        }
                    }
                    break;
            }
        }
        catch (TdbException e)
        {
            System.out.println(e.getDebugMessage());
        }

        return finished;
    }

    /**
     * Main entry point
     * @param args Not used
     */
    public static void main(String[] args)
    {
        try
        {
            Console stdin = new Console();
            Search app = new Search();

            boolean done = false;
            while (!done)
            {
                done = app.handleStatement(stdin.readln("* ", null, false));
            }
            
            app.close();
        }
        catch (TdbException e)
        {
            System.out.println(e.getDebugMessage());
        }
    }
}
