///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2018 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//    TF Apr 09, 2018   Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.database;

import java.util.List;

import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.database.TdbDatabaseClusterDesign;
import com.tietoenator.trip.jxp.database.TdbFieldDesign;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbDateform;
import com.tietoenator.trip.jxp.session.TdbSession;
import com.tietoenator.trip.jxp.TdbControlObject;
import com.tietoenator.trip.jxp.TdbException;

/**
 * Example showing how to retrieve design info on clusters.
 */
public class ShowCluster
{
    private static void printPadding(int size)
    {
        for (int i=0;i<size;i++)
            System.out.print(" ");
    }

    private static void printRightPadded(String str, int size)
    {
        System.out.print(str);
        printPadding(size - str.length());
    }

    private static void printLeftPadded(String str, int size)
    {
        printPadding(size - str.length());
        System.out.print(str);
    }

    /**
     * Main entry point
     * @param args Not used
     */
    public static void main(String[] args)
    {
        TdbSession session = null;
        try
        {
            session = SessionFactory.newInstance("**** Show Cluster ****", false);
            Console stdin = new Console();
            String dbname = "";
            
            while (dbname.length() == 0)
            {
                dbname = stdin.readln("Cluster Name", "");
            }
            
            // What do we want to do with the cluster?
            int option = selectOption(stdin);
            switch (option)
            {
                case 1: // Show std cluster design
                    showStandardClusterDesign(session,stdin,dbname);
                    break;
                case 2: // Show resolved cluster design
                    showResolvedClusterDesign(session,stdin,dbname);
                    break;
                case 3: // Show all database designs
                    showClusterDbDesigns(session,stdin,dbname);
                    break;
                case 4: // Show the fields common to all databases in the cluster
                    showCommonFields(session,stdin,dbname);
                    break;
                default:
                    System.out.println("WARNING: Unhandled option selected");
                    break;
            }

            // Terminate the session
            session.logout();
            session = null;
        }
        catch (Exception e)
        {
            e.printStackTrace();
            if (session != null)
            {
                // Always log out in a controlled fashion
                try { session.logout(); }
                catch (Exception ignore) {}
            }
        }
    }
    
    private static int selectOption(Console stdin)
        throws TdbException
    {
        System.out.println("Select information to show:");
        System.out.println("  [1] Cluster design as stored in CONTROL");
        System.out.println("  [2] Cluster design with all databases resolved");
        System.out.println("  [3] Database designs for all databases in the cluster");
        System.out.println("  [4] Fields common to all databases in the cluster");
        System.out.println();
        
        int option = 0;
        while (option<1 || option>4)
        {
            String optstr = stdin.readln("Option [1-4]","1").trim();
            if (optstr.length()==0)
                return 1;

            try
            {
                option = Integer.parseInt(optstr);
            }
            catch (NumberFormatException x)
            {
                option = 0;
                System.out.println("\nPlease enter an integer between 1 and 4!\n");
            }
        }
        return option;
    }
    
    private static void showStandardClusterDesign(TdbSession session, Console stdin, String name)
    throws Exception
    {
        // Load the cluster design as it is stored in CONTROL
        TdbDatabaseClusterDesign clu = new TdbDatabaseClusterDesign(session);
        long startTime = System.currentTimeMillis();
        clu.get(name);
        long retrTime = System.currentTimeMillis();

        // Display cluster member informtion
        System.out.println();
        System.out.println("STANDARD CLUSTER DESIGN INFORMATION");
        System.out.println();
        printMemberInfo(clu);
        System.out.println("RETRIEVAL TIME: "+Long.toString(retrTime-startTime)+" ms");
        System.out.println("RENDERING TIME: "+Long.toString(System.currentTimeMillis()-retrTime)+" ms");
        System.out.println();
    }
    
    private static void showResolvedClusterDesign(TdbSession session, Console stdin, String name)
    throws Exception
    {
        // Load a "fake" cluster design, with nested clusters
        // resolved into their constituent databases.
        TdbDatabaseClusterDesign clu = new TdbDatabaseClusterDesign(session);
        long startTime = System.currentTimeMillis();
        clu.get(name,true);
        long retrTime = System.currentTimeMillis();

        // Display cluster member informtion
        System.out.println();
        System.out.println("RESOLVED CLUSTER DESIGN INFORMATION");
        System.out.println();
        printMemberInfo(clu);
        System.out.println("RETRIEVAL TIME: "+Long.toString(retrTime-startTime)+" ms");
        System.out.println("RENDERING TIME: "+Long.toString(System.currentTimeMillis()-retrTime)+" ms");
        System.out.println();
    }
    
    private static void printMemberInfo(TdbDatabaseClusterDesign clu)
        throws TdbException
    {
        System.out.println("Member            Type           Records  Creation Date");
        System.out.println("----------------  --------  ------------  --------------------");
        for (TdbControlObject o : clu.members())
        {
            printRightPadded(o.getName(),18);
            if (o.isCluster())
                printRightPadded("CLUSTER",10);
            else if (o.isThesaurus())
                printRightPadded("THESAURUS",10);
            else
                printRightPadded("DATABASE",10);
            
            printLeftPadded(Integer.toString(o.getDatabaseRecordCount()),12);
            printPadding(2);
            
            System.out.print(o.getCreateDate());
            System.out.print(" ");
            System.out.println(o.getCreateTime());
        }
        System.out.println();
    }
    
    private static void showClusterDbDesigns(TdbSession session, Console stdin, String name)
    throws Exception
    {
        // Load the actual database design objects for all databases in the
        // cluster, with nested clusters resolved into their constituent
        // databases.
        TdbDatabaseClusterDesign clu = new TdbDatabaseClusterDesign(session);
        long startTime = System.currentTimeMillis();
        List<TdbDatabaseDesign> dbs = clu.getMemberDesigns(name);
        long retrTime = System.currentTimeMillis();

        // Show selected information on each design
        // (for a more complete example, see ShowDesign.java)
        System.out.println();
        System.out.println("CLUSTERED DATABASE DESIGN INFORMATION");
        System.out.println();
        System.out.println("Name              Type      Fields  Record Count");
        System.out.println("----------------  --------  ------  ------------");
        for (TdbDatabaseDesign db : dbs)
        {
            printRightPadded(db.getName().toUpperCase(),18);
            if (db.isXmlEnabled())
                printRightPadded("XML",10);
            else if (db.isGraph())
                printRightPadded("GRAPH",10);
            else
                printRightPadded("NORMAL",10);
            printLeftPadded(Integer.toString(db.fields().size()),6);
            printPadding(2);
            printLeftPadded(Integer.toString(db.getRecordCount()),12);
            System.out.println();
        }
        System.out.println();
        System.out.println("RETRIEVAL TIME: "+Long.toString(retrTime-startTime)+" ms");
        System.out.println("RENDERING TIME: "+Long.toString(System.currentTimeMillis()-retrTime)+" ms");
        System.out.println();
    }
    
    private static void showCommonFields(TdbSession session, Console stdin, String name)
        throws Exception
    {
        // Load "fake" field design information for fields that are
        // common to all the databases in the cluster.
        TdbDatabaseClusterDesign clu = new TdbDatabaseClusterDesign(session);
        long startTime = System.currentTimeMillis();
        List<TdbFieldDesign> fields = clu.getCommonFields(name);
        long retrTime = System.currentTimeMillis();
        
        // Show selected information on each field.
        // Note that not all information is available for
        // field designs retrieved this way (number, comment, etc).
        // Only the available information is show here.
        System.out.println();
        System.out.println("COMMON CLUSTER FIELDS INFORMATION");
        System.out.println();
        System.out.println("Field Name          Field Type          IsPartField");
        System.out.println("------------------- ------------------- -----------");
        for (TdbFieldDesign fd : fields)
        {
            printRightPadded(fd.getName(),20);
            printRightPadded(fd.getType(),20);
            System.out.print(fd.isPartField() ? "Yes" : "No");
            System.out.println();
        }
        System.out.println();
        System.out.println("RETRIEVAL TIME: "+Long.toString(retrTime-startTime)+" ms");
        System.out.println("RENDERING TIME: "+Long.toString(System.currentTimeMillis()-retrTime)+" ms");
        System.out.println();

    }
}
