///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp programming examples
//
// Copyright (C) 2018 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//        TF    Apr 09, 2018    Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.users;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbControlObject;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbSession;
import com.tietoenator.trip.jxp.control.TdbProcedureList;
import com.tietoenator.trip.jxp.users.TdbProcedure;
import com.tietoenator.trip.jxp.data.TdbSearch;
import com.tietoenator.trip.jxp.data.TdbSearchSet;

/**
 * Example of how to create and use procedures for saved search histories.
 */
public class SearchProcedure
{
    private TdbSession m_session;
    private TdbSearch m_search;
    
    // Simple utility function to print out whatever success messages have been
    // generated by a server transaction
    private void showMsgs(TdbMessageProvider prv)
    {
        for (String msg : prv.messageList())
        {
            System.out.println(msg);
        }
        prv.resetMessages();
    }
    
    /**
     * Constructor
     * @throws TdbException
     */
    public SearchProcedure()
    throws TdbException
    {
        m_session = SessionFactory.newInstance("**** Search Procedure Example ****", false);
    }
    
    public void terminate()
    throws TdbException
    {
        m_session.logout();
    }
    
    private static void printPadding(int size)
    {
        for (int i=0;i<size;i++)
            System.out.print(" ");
    }
    private static void printRightPadded(String str, int size)
    {
        System.out.print(str);
        printPadding(size - str.length());
    }

    private static void printLeftPadded(String str, int size)
    {
        printPadding(size - str.length());
        System.out.print(str);
    }
    
    public void createProcedure(int searchSetNumber, String name, String owner, String comment)
        throws TdbException
    {
        TdbProcedure proc = new TdbProcedure(m_session);
        try
        {
            proc.saveSearch(searchSetNumber,owner,name,comment);
            System.out.println("Procedure saved");
        }
        catch (TdbException x)
        {
            System.out.println("ERROR: "+x.getMessage());
        }
    }
    
    public void executeSearch(String command)
        throws TdbException
    {
        if (null == m_search)
            m_search = new TdbSearch(m_session);
        try
        {
            m_search.execute(command);
            showMsgs(m_search);
            showHistory();
        }
        catch (TdbException x)
        {
            System.out.println("ERROR: "+x.getMessage());
        }
    }
    
    private void showHistory()
    {
        String str;
        for (TdbSearchSet ss : m_search)
        {
            str = "S=" + Integer.toString(ss.getSearchId());
            printRightPadded(str,10);
            str = "<" + Integer.toString(ss.getRecordCount()) + ">";
            printRightPadded(str,14);
            System.out.println(ss.getParsedCommand());
        }
    }

    public void showProcedureText(String owner, String name)
    throws TdbException
    {
        TdbProcedure proc = new TdbProcedure(m_session);
        proc.get(owner,name);
        for (String line : proc)
            System.out.println(line);
    }

    public void deleteProcedure(String owner, String name)
    throws TdbException
    {
        TdbProcedure proc = new TdbProcedure(m_session);
        proc.delete(owner,name);
        showMsgs(proc);
    }

    public void listProcedures(String owner)
    throws TdbException
    {
        TdbProcedureList proclist = new TdbProcedureList(m_session,owner);
        for (TdbControlObject o : proclist)
        {
            String id = o.getOwnerName() + "." + o.getName();
            printRightPadded(id,34);
            System.out.println(o.getComment());
        }
    }
    
    private static void printMenu()
    {
        System.out.println();
        System.out.println("1) Execute CCL command");
        System.out.println("2) Show search history");
        System.out.println("3) Create procedure for search set");
        System.out.println("4) List proceures");
        System.out.println("5) Show procedure text");
        System.out.println("6) Execute search procedure");
        System.out.println("7) Delete procedure");
        System.out.println("9) Exit");
    }
    
    public static void main(String[] args)
    {
        try
        {
            SearchProcedure prg = new SearchProcedure();
            Console stdin = new Console();
            boolean quit = false;
            String name, owner, ccl;
            
            printMenu();
            for (; !quit;)
            {
                try
                {
                    switch (stdin.readint("\nChoose function", 0, true))
                    {
                        case 1:
                            ccl = stdin.readln("CCL search command (BASE, FIND or FUZZ): ", "");
                            if (ccl.length() > 0)
                                prg.executeSearch(ccl);
                            break;
                            
                        case 2:
                            prg.showHistory();
                            break;
                            
                        case 3:
                            name = stdin.readln("Procedure Name","");
                            if (name.length() == 0)
                            {
                                System.out.println("No name specified. Aborting...");
                                break;
                            }
                            owner = stdin.readln("Owner",prg.m_session.getUsername());
                            int searchSet = stdin.readint("Search Set #",0,true);
                            String cmt = stdin.readln("Comment","");
                            prg.createProcedure(searchSet,name,owner,cmt);
                            break;
                            
                        case 4:
                            owner = stdin.readln("User/Group","");
                            prg.listProcedures(owner);
                            break;
                            
                        case 5:
                            name = stdin.readln("Procedure Name", "");
                            if (name.length() == 0)
                            {
                                System.out.println("No name specified. Aborting...");
                                break;
                            }
                            owner = stdin.readln("Owner",prg.m_session.getUsername());
                            prg.showProcedureText(owner,name);
                            break;
                            
                        case 6:
                            name = stdin.readln("Procedure Name", "");
                            if (name.length() == 0)
                            {
                                System.out.println("No name specified. Aborting...");
                                break;
                            }
                            prg.executeSearch("FIND SAVE "+name);
                            break;
                            
                        case 7:
                            name = stdin.readln("Procedure Name", "");
                            if (name.length() == 0)
                            {
                                System.out.println("No name specified. Aborting...");
                                break;
                            }
                            owner = stdin.readln("Owner",prg.m_session.getUsername());
                            prg.deleteProcedure(owner,name);
                            break;
                        case 9:
                            quit = true;
                            break;
                        default:
                            printMenu();
                            break;
                    }
                }
                catch (Exception e)
                {
                    System.out.println(e.getMessage());
                }
            }
            prg.terminate();
        }
        catch (TdbException e)
        {
            System.out.println(e.getDebugMessage());
        }
    }
}

